function Z = mtn(n)
% mtn (hyperbolic network model)
% by Christian Himpe, 2013 ( http://gramian.de )
% released under BSD 2-Clause License ( opensource.org/licenses/BSD-2-Clause )
%*

if(exist('emgr')~=2) disp('emgr framework is required. Download at http://gramian.de/emgr.m'); return; end

%%%%%%%% Setup %%%%%%%%

 J = n;
 O = J;
 N = n*n;
 R = O;
 T = [0 0.01 1];
 L = (T(3)-T(1))/T(2);
 U = [ones(J,1) zeros(J,L-1)];
 X =  zeros(N,1);

 rand('seed',1009)
 A = sprand(N,N,0.2); A(1:N+1:end) = -0.12*N; A = 0.5*(A+A');
 B = rand(N,J);
 C = B';
 P = A(:);
 Q = -eye(N); Q = Q(:);

 NON = @(x,u,p) reshape(p,[N N])*tanh(x) + B*u;
 OUT = @(x,u,p) C*x;

 Z = zeros(8,3);
 fprintf('Progress: ');

 warning off all;
 OPT = optimset('Display','off');

 norm2  = @(y) sqrt(sum(sum(y.*y)));
 normp  = @(y) sum(y.*y);
 norm22 = @(y) sum(sum(y.*y));



%%%%%%%% Nonlinear Combined Reduction (Inverse) %%%%%%%%

% FULL
 tic; Y = rk2(NON,OUT,T,X,U,P); Z(1,:) = [NaN,toc,NaN];

 Y = Y + 0.001*rand(O,L);

% FULL
 tic;
 j = @(q,o) norm22(Y-rk2(NON,OUT,T,X,U,q));
 r = fminunc(j,Q,OPT);
 y = rk2(NON,OUT,T,X,U,r);
 Z(5,:) = [NaN,toc,norm2(norm2(Y - y)./norm2(Y))];
 fprintf('#');


% WSWO OFFLINE
 tic;
 WS = emgr(NON,OUT,[J N O],T,'s',Q,0,1,0,1);
 WO = emgr(NON,OUT,[J N O],T,'o',Q,0,1,0,1);
 [UU D VV] = balance(WS{1},WO);
 [PP D QQ] = svd(WS{2});
 UU = UU(1:R,:);   VV = VV(:,1:R);
 PP = PP(:,1:R*R); QQ = QQ(:,1:R*R)';
 x = UU*X;
 q = QQ*Q;
 non = @(x,u,p) UU*NON(VV*x,u,PP*p);
 out = @(x,u,p) OUT(VV*x,u,PP*p);
 Z(6,1) = toc;

% WSWO ONLINE
 tic;
 j = @(q,o) norm22(Y-rk2(non,out,T,x,U,q));
 r = fminunc(j,q,OPT);
 y = rk2(non,out,T,x,U,r);
 Z(6,2) = toc; Z(6,3) = norm2(Y - y)./norm2(Y);
 fprintf('#');


% WCWI OFFLINE
 tic;
 WI = emgr(NON,OUT,[J N O],T,'i',Q,0,1,0,1);
 WC = emgr(NON,OUT,[J N O],T,'c',Q,0,1,0,1);
 [UU D VV] = balance(WC,WI{1});
 [PP D QQ] = svd(WI{2});
 UU = UU(1:R,:);   VV = VV(:,1:R);
 PP = PP(:,1:R*R); QQ = QQ(:,1:R*R)';
 x = UU*X;
 q = QQ*Q;
 non = @(x,u,p) UU*NON(VV*x,u,PP*p);
 out = @(x,u,p) OUT(VV*x,u,PP*p);
 Z(7,1) = toc;

% WCWI ONLINE
 tic;
 j = @(q,o) norm22(Y-rk2(non,out,T,x,U,q));
 r = fminunc(j,q,OPT);
 y = rk2(non,out,T,x,U,r);
 Z(7,2) = toc; Z(7,3) = norm2(Y - y)./norm2(Y);
 fprintf('#');


% WJ OFFLINE
 tic;
 WJ = emgr(NON,OUT,[J N O],T,'j',Q,0,1,0,1);
 [UU D VV] = svd(WJ{1});
 [PP D QQ] = svd(WJ{2});
 UU = UU(:,1:R);   VV = VV(:,1:R)';
 PP = PP(1:R*R,:); QQ = QQ(1:R*R,:)';
 x = VV*X;
 q = PP*Q;
 non = @(x,u,p) VV*NON(UU*x,u,QQ*p);
 out = @(x,u,p) OUT(UU*x,u,QQ*p);
 Z(8,1) = toc;

% WJ ONLINE
 tic;
 j = @(q,o) norm22(Y-rk2(non,out,T,x,U,q));
 r = fminunc(j,q,OPT);
 y = rk2(non,out,T,x,U,r);
 Z(8,2) = toc; Z(8,3) = norm2(Y - y)./norm2(Y);
 fprintf('#\n');


%%%%%%%% Integrator %%%%%%%%

function y = rk2(f,g,t,x,u,p)

 h = t(2);
 T = (t(3)-t(1))/h;
 y = zeros(numel(g(x,u(:,1),p)),T);

 for t=1:T
  x = x + h*f(x + 0.5*h*f(x,u(:,t),p),u(:,t),p); %Improved Eulers Method
  y(:,t) = g(x,u(:,t),p);
 end

%%%%%%%% Balancer %%%%%%%%

function [X Y Z] = balance(WC,WO)

 L = chol(WC+eye(size(WC,1)))-eye(size(WC,1));
 [U Y V] = svd(L*WO*L');
 X = diag(sqrt(diag(Y))) * V' / L';
 Z = L'*U*diag(1./sqrt(diag(Y))); 
